@@ -7,6 +7,7 @@ pub mod optimise;
 use std::{
     cmp::{Ordering, Reverse},
     collections::{BinaryHeap, HashMap, HashSet},
+    ops::Index,
     sync::Mutex,
 };
 
@@ -40,11 +41,13 @@ pub struct Graph {
     // that isn't actually in the graph - in this case they will be ignored or discarded during the
     // optimisation process).
     chunks: HashMap<ChunkId, Chunk>,
+    links: LinkSet,
+
     /// **Invariant**: If `start_chunks` points to a chunk, it **must** be a start chunk (i.e. not
     /// have any predecessors, and have `start_label` set)
     start_chunks: Vec<(ChunkId, StartIdx, Rotation)>,
-    /// **Invariant**: If `start_chunks` points to a chunk, it **must** be a end chunk (i.e. not have
-    /// any successors, and have `end_chunks` set)
+    /// **Invariant**: If `start_chunks` points to a chunk, it **must** be a end chunk (i.e. not
+    /// have any successors, and have `end_chunks` set)
     end_chunks: Vec<(ChunkId, End)>,
     /// The number of different parts
     num_parts: Rotation,
@@ -64,8 +67,8 @@ pub struct Chunk {
     /// The string that should be added when this chunk is generated
     label: String,
 
-    successors: Vec<Link>,
-    predecessors: Vec<Link>,
+    successors: Vec<LinkId>,
+    predecessors: Vec<LinkId>,
 
     /// The chunks which share rows with `self`, including `self` (because all chunks are false
     /// against themselves).  Optimisation passes probably shouldn't mess with falseness.
@@ -102,21 +105,101 @@ pub struct Chunk {
     pub lb_distance_to_rounds: Distance,
 }
 
+//////////
+// LINK //
+//////////
+
+/// A link between two [`Chunk`]s in a [`Graph`]
 #[derive(Debug, Clone, Eq, PartialEq, Hash)]
 pub struct Link {
-    pub id: ChunkId,
+    pub from: ChunkId,
+    pub to: ChunkId,
     /// Indexes into `Layout::links`
     pub source_idx: LinkIdx,
     pub rotation: Rotation,
+    pub rotation_back: Rotation,
 }
 
 impl Link {
-    pub fn new(id: ChunkId, source_idx: LinkIdx, rotation: Rotation) -> Self {
-        Self {
-            id,
-            source_idx,
-            rotation,
-        }
+    /// Computes the [`Score`] created by this link **in one part**
+    pub fn weight_per_part(&self, from: &ChunkId, query: &Query) -> f32 {
+        let source = &query.layout.links[self.source_idx];
+        let is_splice = match (from, &self.to) {
+            (ChunkId::ZeroLengthEnd, _) => {
+                unreachable!("Can't have link out of 0-length end node");
+            }
+            // Going to a 0-length end is **not** a splice
+            (ChunkId::Standard(_), ChunkId::ZeroLengthEnd) => false,
+            // A link between two full chunks is a splice iff it changes the block
+            (ChunkId::Standard(std1), ChunkId::Standard(std2)) => {
+                std1.row_idx.block != std2.row_idx.block
+            }
+        };
+
+        let call_weight = match source.call_idx {
+            Some(idx) => query.calls[idx].weight,
+            None => 0.0, // Plain leads have no weight
+        };
+        let splice_weight = if is_splice { query.splice_weight } else { 0.0 };
+        call_weight + splice_weight
+    }
+}
+
+/// Unique identifier for a [`Link`] within a [`Graph`]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub struct LinkId(usize);
+
+/// A [`HashMap`] containing a set of [`Link`]s, all addressed by unique [`LinkId`]s
+#[derive(Debug, Clone, Default)]
+pub struct LinkSet {
+    next_id: usize,
+    map: HashMap<LinkId, Link>,
+}
+
+impl LinkSet {
+    /// Create a [`LinkSet`] containing no [`Link`]s
+    pub fn new() -> Self {
+        Self::default()
+    }
+
+    /// Add a new [`Link`] to this set, returning its [`LinkId`]
+    pub fn add(&mut self, link: Link) -> LinkId {
+        let id = self.next_id();
+        self.map.insert(id, link);
+        id
+    }
+
+    pub fn get(&self, id: LinkId) -> Option<&Link> {
+        self.map.get(&id)
+    }
+
+    pub fn get_mut(&mut self, id: LinkId) -> Option<&mut Link> {
+        self.map.get_mut(&id)
+    }
+
+    pub fn keys(&self) -> std::iter::Copied<std::collections::hash_map::Keys<LinkId, Link>> {
+        self.map.keys().copied()
+    }
+
+    /// Remove any [`Link`]s from `self` which don't satisfy a given predicate
+    pub fn retain(&mut self, mut pred: impl FnMut(LinkId, &mut Link) -> bool) {
+        self.map.retain(|id, link| pred(*id, link))
+    }
+
+    /// Get a new [`LinkId`], unique from all the others returned from `self`
+    fn next_id(&mut self) -> LinkId {
+        let id = LinkId(self.next_id);
+        self.next_id += 1;
+        id
+    }
+}
+
+impl Index<LinkId> for LinkSet {
+    type Output = Link;
+
+    #[track_caller]
+    fn index(&self, index: LinkId) -> &Self::Output {
+        &self.map[&index]
     }
 }
 
@@ -126,7 +209,8 @@ impl Graph {
     //! Optimisation
 
     /// Repeatedly apply a sequence of [`Pass`]es until the graph stops getting smaller, or 20
-    /// iterations are made.  Use [`Graph::optimise_with_iter_limit`] to set a custom iteration limit.
+    /// iterations are made.  Use [`Graph::optimise_with_iter_limit`] to set a custom iteration
+    /// limit.
     pub fn optimise(&mut self, passes: &[Mutex<Pass>], query: &Query) {
         self.optimise_with_iter_limit(passes, query, 20);
     }
@@ -139,19 +223,42 @@ impl Graph {
         query: &Query,
         limit: usize,
     ) {
-        let mut last_size = Size::from(&*self);
-
-        for _ in 0..limit {
+        let mut last_size = self.size();
+        log::debug!("Initial graph size: {:?}", last_size);
+        let mut iter_count = 0;
+        loop {
+            // Run every optimisation pass
             for p in passes {
+                // TODO: Find a better locking system, or remove the `FnMut` bound so that locking
+                // is unnecessary.  I think that this system can deadlock if multiple threads are
+                // optimising graphs in parallel using the same set of passes.
                 p.lock().unwrap().run(self, query);
             }
-
-            let new_size = Size::from(&*self);
-            // Stop optimising if the optimisations don't make the graph strictly smaller.  If
-            // they make some parts smaller but others larger, then keep optimising.
-            match new_size.partial_cmp(&last_size) {
-                Some(Ordering::Equal | Ordering::Greater) => return,
-                Some(Ordering::Less) | None => {}
+            // Stop optimising if the limit has been reached
+            if iter_count > limit {
+                log::warn!(
+                    "Graph optimisation limit reached, but more progress could have been made."
+                );
+                break;
+            }
+            iter_count += 1;
+            // Stop optimising if the graph has stopped getting smaller
+            let new_size = self.size();
+            log::debug!("New size: {:?}", new_size);
+            match new_size.cmp(&last_size) {
+                // If graph got smaller, then keep optimising in case more optimisation is possible
+                Ordering::Less => {}
+                // If the last optimisation pass couldn't make `self` smaller, then assume no
+                // changes has been made and further optimisation is impossible
+                Ordering::Equal => {
+                    log::debug!(
+                        "Finished optimisation after {} iters of every pass",
+                        iter_count
+                    );
+                    break;
+                }
+                // Optimisation passes shouldn't increase the graph size
+                Ordering::Greater => unreachable!("Optimisation should never increase graph size"),
             }
             last_size = new_size;
         }
@@ -175,6 +282,24 @@ impl Graph {
     pub fn num_parts(&self) -> Rotation {
         self.num_parts
     }
+
+    /// Return a value representing the 'size' of this graph.  Optimisation passes are
+    /// **required** to never increase this quantity.  Graph size is compared on the following
+    /// factors (in order of precedence, most important first):
+    /// 1. Number of nodes (smaller is better)
+    /// 2. Number of links (smaller is better)
+    /// 3. Number of required nodes (more is better)
+    pub fn size(&self) -> (usize, usize, Reverse<usize>) {
+        let mut num_links = 0;
+        let mut num_required_nodes = 0;
+        for chunk in self.chunks.values() {
+            num_links += chunk.successors.len();
+            if chunk.required {
+                num_required_nodes += 1;
+            }
+        }
+        (self.chunks.len(), num_links, Reverse(num_required_nodes))
+    }
 }
 
 // ------------------------------------------------------------------------------------------
@@ -187,6 +312,20 @@ impl Graph {
         self.chunks.retain(pred);
     }
 
+    /// Removes all links for whom `pred` returns `false`.  The parameters of `pred` are
+    /// `(link, chunk_from, chunk_to)`
+    pub fn retain_links(
+        &mut self,
+        mut pred: impl FnMut(&Link, &ChunkId, &Chunk, &ChunkId, &Chunk) -> bool,
+    ) {
+        self.links.retain(|_id, link| {
+            match (self.chunks.get(&link.from), self.chunks.get(&link.to)) {
+                (Some(from), Some(to)) => pred(link, &link.from, from, &link.to, to),
+                _ => false, // If either side of the link is missing, remove the link
+            }
+        })
+    }
+
     /// Remove elements from [`Self::start_chunks`] for which a predicate returns `false`.
     pub fn retain_start_chunks(
         &mut self,
@@ -204,6 +343,14 @@ impl Graph {
 impl Chunk {
     //! Helpers for optimisation passes
 
+    /// Returns the mutual truth of `self` against the chunk with id of `other`
+    pub fn truth_against(&self, other: &ChunkId) -> Truth {
+        Truth::from(match other.std_id() {
+            Some(std_id) => !self.false_chunks.contains(std_id),
+            None => true, // All chunks are true against 0-length chunks
+        })
+    }
+
     /// A lower bound on the length of a composition which passes through this chunk.
     pub fn min_comp_length(&self) -> usize {
         self.lb_distance_from_rounds + self.length() + self.lb_distance_to_rounds
@@ -219,54 +366,6 @@ impl Chunk {
     }
 }
 
-/// A measure of the `Size` of a [`Graph`].  Used to detect when further optimisations aren't
-/// useful.
-#[derive(Debug, PartialEq, Clone, Copy)]
-struct Size {
-    num_chunks: usize,
-    num_links: usize,
-    num_starts: usize,
-    num_ends: usize,
-}
-
-impl From<&Graph> for Size {
-    fn from(g: &Graph) -> Self {
-        Self {
-            num_chunks: g.chunks.len(),
-            // This assumes that every successor link also corresponds to a predecessor link
-            num_links: g.chunks().map(|(_id, chunk)| chunk.successors.len()).sum(),
-            num_starts: g.start_chunks.len(),
-            num_ends: g.end_chunks.len(),
-        }
-    }
-}
-
-impl PartialOrd for Size {
-    // TODO: Make this into a macro?
-    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        let cmp_chunks = self.num_chunks.cmp(&other.num_chunks);
-        let cmp_links = self.num_links.cmp(&other.num_links);
-        let cmp_starts = self.num_starts.cmp(&other.num_starts);
-        let cmp_ends = self.num_ends.cmp(&other.num_ends);
-
-        let all_comparisons = [cmp_chunks, cmp_links, cmp_starts, cmp_ends];
-
-        let are_any_less = all_comparisons
-            .iter()
-            .any(|cmp| matches!(cmp, Ordering::Less));
-        let are_any_greater = all_comparisons
-            .iter()
-            .any(|cmp| matches!(cmp, Ordering::Greater));
-
-        match (are_any_less, are_any_greater) {
-            (true, false) => Some(Ordering::Less), // If nothing got larger, then the size is smaller
-            (false, true) => Some(Ordering::Greater), // If nothing got smaller, then the size is larger
-            (false, false) => Some(Ordering::Equal),  // No < or > means all components are equal
-            (true, true) => None, // If some are smaller & some are greater then these are incomparable
-        }
-    }
-}
-
 // ------------------------------------------------------------------------------------------
 
 impl Graph {
@@ -274,6 +373,14 @@ impl Graph {
 
     // Getters
 
+    pub fn get_link(&self, id: LinkId) -> Option<&Link> {
+        self.links.get(id)
+    }
+
+    pub fn get_link_mut(&mut self, id: LinkId) -> Option<&mut Link> {
+        self.links.get_mut(id)
+    }
+
     pub fn get_chunk<'graph>(&'graph self, id: &ChunkId) -> Option<&'graph Chunk> {
         self.chunks.get(id)
     }
@@ -367,19 +474,19 @@ impl Chunk {
 
     // CROSS-CHUNK REFERENCES //
 
-    pub fn successors(&self) -> &[Link] {
+    pub fn successors(&self) -> &[LinkId] {
         self.successors.as_slice()
     }
 
-    pub fn successors_mut(&mut self) -> &mut Vec<Link> {
+    pub fn successors_mut(&mut self) -> &mut Vec<LinkId> {
         &mut self.successors
     }
 
-    pub fn predecessors(&self) -> &[Link] {
+    pub fn predecessors(&self) -> &[LinkId] {
         self.predecessors.as_slice()
     }
 
-    pub fn predecessors_mut(&mut self) -> &mut Vec<Link> {
+    pub fn predecessors_mut(&mut self) -> &mut Vec<LinkId> {
         &mut self.predecessors
     }
 
@@ -426,6 +533,7 @@ impl Graph {
 
         // Convert each `expanded_chunk_range` into a full `Chunk`, albeit without
         // predecessor/falseness references
+        let mut links = LinkSet::new();
         let mut chunks: HashMap<ChunkId, Chunk> = expanded_chunk_ranges
             .iter()
             .map(|(chunk_id, (chunk_range, distance))| {
@@ -445,12 +553,14 @@ impl Graph {
                 let new_chunk = build_chunk(
                     chunk_range,
                     *distance,
-                    // We've asserted that all chunks have even length, so that all chunks must start
-                    // at the same stroke.  Therefore, we can simply pass 'start_stroke' straight
-                    // to all the comps
+                    num_parts,
+                    // We've asserted that all chunks have even length, so that all chunks must
+                    // start at the same stroke.  Therefore, we can simply pass 'start_stroke'
+                    // straight to all the comps
                     query.start_stroke,
                     &part_heads,
                     query,
+                    &mut links,
                 );
                 (chunk_id.clone(), new_chunk)
             })
@@ -475,16 +585,9 @@ impl Graph {
         log::debug!("Setting predecessor links");
         for (id, _dist) in expanded_chunk_ranges {
             if let Some(chunk) = chunks.get(&id) {
-                for succ_link in chunk.successors.clone() {
-                    if let Some(succ_chunk) = chunks.get_mut(&succ_link.id) {
-                        assert!(succ_link.rotation < num_parts);
-                        succ_chunk.predecessors.push(Link {
-                            id: id.clone(),
-                            source_idx: succ_link.source_idx,
-                            // Passing backwards over a link gives it the opposite rotation to
-                            // traversing forward
-                            rotation: num_parts - succ_link.rotation,
-                        });
+                for succ_link_id in chunk.successors.clone() {
+                    if let Some(succ_chunk) = chunks.get_mut(&links[succ_link_id].to) {
+                        succ_chunk.predecessors.push(succ_link_id);
                     }
                 }
             }
@@ -492,6 +595,7 @@ impl Graph {
 
         Ok(Self {
             chunks,
+            links,
             start_chunks,
             end_chunks,
             num_parts,
@@ -499,8 +603,8 @@ impl Graph {
     }
 }
 
-/// Given an initial set of chunks, compute the pairs of false chunks and remove any chunks which are
-/// self-false.
+/// Given an initial set of chunks, compute the pairs of false chunks and remove any chunks which
+/// are self-false.
 fn compute_falseness(
     chunks: &mut HashMap<ChunkId, Chunk>,
     layout: &Layout,
@@ -667,9 +771,11 @@ fn build_graph(query: &Query, size_limit: usize) -> Result<BuiltGraph, BuildErro
 fn build_chunk(
     chunk_range: &ChunkRange,
     distance: usize,
+    num_parts: Rotation,
     start_stroke: Stroke,
     part_heads: &[RowBuf],
     query: &Query,
+    link_set: &mut LinkSet,
 ) -> Chunk {
     // Add up music from each part
     let mut music = Breakdown::zero(query.music_types.len());
@@ -686,7 +792,8 @@ fn build_chunk(
             // Count weight from CH masks
             for (mask, weight) in &query.ch_weights {
                 if mask.matches(&ch) {
-                    music.score += *weight * chunk_range.total_length.0 as f32; // Weight applies to each row
+                    // Weight applies to each row
+                    music.score += *weight * chunk_range.per_part_length.0 as f32;
                 }
             }
         }
@@ -702,6 +809,22 @@ fn build_chunk(
         .zip_eq(music.counts.as_slice())
         .any(|(music_type, count)| music_type.non_duffer() && *count > 0);
 
+    let successors = chunk_range
+        .links()
+        .iter()
+        .cloned()
+        .map(|(source_idx, to, rotation)| {
+            assert!(rotation < num_parts);
+            link_set.add(Link {
+                from: chunk_range.chunk_id.clone(),
+                to,
+                source_idx,
+                rotation,
+                rotation_back: num_parts - rotation,
+            })
+        })
+        .collect_vec();
+
     Chunk {
         per_part_length: chunk_range.per_part_length,
         total_length: chunk_range.total_length,
@@ -724,12 +847,7 @@ fn build_chunk(
         // so we can set it to 0 without breaking any invariants.
         lb_distance_to_rounds: 0,
 
-        successors: chunk_range
-            .links()
-            .iter()
-            .cloned()
-            .map(|(idx, id, rotation)| Link::new(id, idx, rotation))
-            .collect_vec(),
+        successors,
 
         // These are populated in separate passes once all the `Chunk`s have been created
         false_chunks: Vec::new(),
