@@ -3,13 +3,13 @@ use std::{
     ops::Mul,
 };
 
-use bellframe::{Mask, Row, RowBuf, Stage};
+use bellframe::{Bell, Mask, Row, RowBuf, Stage};
 use index_vec::IndexVec;
 
-use super::{utils::CourseHeadMask, Boundary, Error, Result, SpliceStyle};
+use super::{utils::CourseHeadMask, Boundary, Error, Result};
 use crate::{
     layout::{BlockIdx, BlockVec, Layout, Link, RowIdx, StartOrEnd},
-    CallIdx, CallVec,
+    CallIdx, CallVec, SpliceStyle,
 };
 
 /// Generate a [`Layout`] such that chunks will be labelled by their course-head, rather than by
@@ -17,7 +17,9 @@ use crate::{
 pub fn coursewise(
     mut methods: Vec<super::Method>,
     calls: &CallVec<super::Call>,
+    calling_bell: Bell,
     splice_style: SpliceStyle,
+    part_head: &Row,
 ) -> Result<Layout> {
     // Cache data about each method, and compute the overall stage of the comp
     super::utils::check_duplicate_shorthands(&methods)?;
@@ -32,6 +34,7 @@ pub fn coursewise(
     // Pre-process CH masks
     super::utils::preprocess_ch_masks(&mut methods, calls, stage)?;
     for m in &methods {
+        check_chs_in_other_parts(&m.ch_masks, part_head)?;
         super::utils::check_for_ambiguous_courses(
             &m.ch_masks,
             &m.lead_head().closure_from_rounds(),
@@ -39,7 +42,7 @@ pub fn coursewise(
     }
 
     // Generate links
-    let links = generate_links(&methods, calls, stage, splice_style)?.into();
+    let links = generate_links(&methods, calls, calling_bell, stage, splice_style)?.into();
 
     Ok(Layout {
         links,
@@ -55,6 +58,29 @@ pub fn coursewise(
     })
 }
 
+/// Check that the set of CH masks is preserved by the part head.
+fn check_chs_in_other_parts(chs: &[CourseHeadMask], part_head: &Row) -> Result<()> {
+    for part_head in part_head.closure() {
+        'ch_loop: for mask in chs {
+            let mask_in_other_part = part_head.mul(mask.mask());
+            // Check if there's an entry in `chs` which matches the `mask_in_other_part`
+            for mask2 in chs {
+                if mask_in_other_part.is_subset_of(mask2.mask()) {
+                    continue 'ch_loop; // `mask2` contains `transposed_mask`, so we're good
+                }
+            }
+            // If no matching CH mask was found, then `mask` doesn't have any matching entry in
+            // the part starting with `ph`
+            return Err(Error::NoCourseHeadInPart {
+                mask_in_first_part: mask.mask().to_owned(),
+                part_head,
+                mask_in_other_part,
+            });
+        }
+    }
+    Ok(()) // If we didn't find a mismatch, then the CHs are OK
+}
+
 /////////////////////
 // LINK GENERATION //
 /////////////////////
@@ -63,6 +89,7 @@ pub fn coursewise(
 fn generate_links(
     methods: &[super::Method],
     calls: &CallVec<super::Call>,
+    calling_bell: Bell,
     stage: Stage,
     mut splice_style: SpliceStyle,
 ) -> Result<Vec<Link>> {
@@ -85,6 +112,7 @@ fn generate_links(
         calls,
         &call_ends,
         &call_starts_by_label,
+        calling_bell,
         stage,
         splice_style,
     )?;
@@ -191,6 +219,7 @@ fn generate_all_links(
     calls: &CallVec<super::Call>,
     call_ends: &[CallEnd],
     call_starts_by_label: &HashMap<&str, Vec<RowIdx>>,
+    calling_bell: Bell,
     stage: Stage,
     splice_style: SpliceStyle,
 ) -> Result<Vec<Link>> {
@@ -217,6 +246,7 @@ fn generate_all_links(
                 method_idx,
                 method_data,
                 calls,
+                calling_bell,
                 from_ch_mask,
                 &link_gen_data,
                 &mut links,
@@ -237,6 +267,7 @@ fn generate_call_links(
     method_idx: usize,
     m: &super::Method,
     calls: &CallVec<super::Call>,
+    calling_bell: Bell,
     from_ch_mask: &CourseHeadMask,
     link_gen_data: &LinkGenData,
     links: &mut Vec<Link>,
@@ -267,7 +298,7 @@ fn generate_call_links(
 
             // Get the debug/display names for any link in this position
             let tenor_place = mask_after_call
-                .place_of(from_ch_mask.calling_bell())
+                .place_of(calling_bell)
                 .expect("Course head mask doesn't fix the calling bell");
             let calling_position = call.calling_positions.get(tenor_place).ok_or_else(|| {
                 Error::CallingPositionsTooShort {
