@@ -5,11 +5,10 @@ use std::{
     ops::Range,
 };
 
-use bellframe::{method::RowAnnot, Bell, Block, Mask, PlaceNot, Row, Stage};
+use bellframe::{method::RowAnnot, Bell, Block, Mask, PlaceNot, Row, RowBuf, Stage};
 use itertools::Itertools;
-use serde::Deserialize;
 
-use crate::CallVec;
+use crate::{CallVec, SpliceStyle};
 
 use super::{Layout, MethodBlock};
 
@@ -21,23 +20,19 @@ impl Layout {
     pub fn new(
         methods: Vec<self::Method>,
         calls: &CallVec<self::Call>,
+        calling_bell: Bell,
         splice_style: SpliceStyle,
         part_head: &Row,
         leadwise: Option<bool>,
     ) -> Result<Self> {
-        let leadwise = leadwise.unwrap_or_else(|| {
-            // Set 'coursewise' as the default iff the part head doesn't preserve the positions of
-            // all calling bells
-            !methods
-                .iter()
-                .flat_map(|m| &m.ch_masks)
-                .all(|ch_mask| part_head.is_fixed(ch_mask.calling_bell()))
-        });
+        // Set 'coursewise' as the default iff the part head doesn't preserve the positions of
+        // the calling bell
+        let leadwise = leadwise.unwrap_or_else(|| !part_head.is_fixed(calling_bell));
 
         if leadwise {
-            leadwise::leadwise(&methods, calls)
+            leadwise::leadwise(&methods, calls, splice_style)
         } else {
-            coursewise::coursewise(methods, calls, splice_style)
+            coursewise::coursewise(methods, calls, calling_bell, splice_style, part_head)
         }
     }
 }
@@ -60,6 +55,11 @@ pub enum Error {
         calling_position_len: usize,
         stage: Stage,
     },
+    NoCourseHeadInPart {
+        mask_in_first_part: Mask,
+        part_head: RowBuf,
+        mask_in_other_part: Mask,
+    },
     /// Some courses match two different CH masks which specify **different** calling bells.
     ConflictingCallingBell((Mask, Bell), (Mask, Bell)),
     AmbiguousCourseHeadPosition {
@@ -92,6 +92,21 @@ impl Display for Error {
                 "Call {:?} refers to a lead location {:?}, which doesn't exist",
                 call_name, label
             ), // TODO: Suggest one that does exist
+            Self::NoCourseHeadInPart {
+                mask_in_first_part,
+                part_head,
+                mask_in_other_part,
+            } => {
+                writeln!(f,
+                    "course head `{}` becomes `{}` in the part starting `{}`, which isn't in `course_heads`.",
+                    mask_in_first_part, mask_in_other_part, part_head
+                )?;
+                write!(
+                    f,
+                    "   help: consider adding `{}` to `course_heads`",
+                    mask_in_other_part
+                )
+            }
             Self::CallingPositionsTooShort {
                 call_name,
                 calling_position_len,
@@ -132,40 +147,19 @@ impl std::error::Error for Error {}
 
 pub type Result<T> = std::result::Result<T, Error>;
 
-/// The different styles of spliced that can be generated
-#[derive(Debug, Clone, Copy, Eq, PartialEq, Hash, Deserialize)]
-pub enum SpliceStyle {
-    /// Splices could happen at any lead label
-    #[serde(rename = "leads")]
-    LeadLabels,
-    /// Splice only happen whenever a call _could_ have happened
-    #[serde(rename = "call locations")]
-    CallLocations,
-    /// Splices only happen when calls are actually made
-    #[serde(rename = "calls")]
-    Calls,
-}
-
-impl Default for SpliceStyle {
-    fn default() -> Self {
-        Self::LeadLabels
-    }
-}
-
 #[derive(Debug, Clone)]
 pub enum CourseHeadMaskPreset {
     TenorsTogether,
     SplitTenors,
-    Custom(Vec<(Mask, Bell)>),
+    Custom(Vec<Mask>),
 }
 
 impl CourseHeadMaskPreset {
-    pub fn into_masks(self, stage: Stage) -> Vec<(Mask, Bell)> {
-        let tenor = stage.tenor();
+    pub fn into_masks(self, stage: Stage) -> Vec<Mask> {
         match self {
-            Self::TenorsTogether => vec![(tenors_together_mask(stage), tenor)],
+            Self::TenorsTogether => vec![tenors_together_mask(stage)],
             // Only fix the tenor for split tenors comps
-            Self::SplitTenors => vec![(Mask::fix_bells(stage, vec![tenor]), tenor)],
+            Self::SplitTenors => vec![Mask::fix_bells(stage, vec![stage.tenor()])],
             Self::Custom(ch_masks) => ch_masks,
         }
     }
@@ -218,7 +212,8 @@ pub struct Method {
 impl Method {
     pub fn new(
         method: bellframe::Method,
-        ch_masks: Vec<(Mask, Bell)>,
+        ch_masks: Vec<Mask>,
+        calling_bell: Bell,
         shorthand: String,
         count_range: Range<usize>,
         start_indices: Option<&[isize]>,
@@ -242,7 +237,7 @@ impl Method {
             count_range,
             ch_masks: ch_masks
                 .into_iter()
-                .flat_map(|(mask, calling_bell)| utils::CourseHeadMask::new(mask, calling_bell))
+                .flat_map(|mask| utils::CourseHeadMask::new(mask, calling_bell))
                 .collect_vec(),
         }
     }
