@@ -9,6 +9,7 @@ pub mod music;
 mod search;
 pub mod utils;
 
+use serde::Deserialize;
 pub use utils::OptRange;
 
 use itertools::Itertools;
@@ -40,6 +41,7 @@ pub struct Query {
     pub len_range: Range<usize>,
     pub num_comps: usize,
     pub allow_false: bool,
+    pub splice_style: SpliceStyle,
 
     pub calls: CallVec<CallType>,
     pub part_head: RowBuf,
@@ -62,6 +64,26 @@ impl Query {
     }
 }
 
+/// The different styles of spliced that can be generated
+#[derive(Debug, Clone, Copy, Eq, PartialEq, Hash, Deserialize)]
+pub enum SpliceStyle {
+    /// Splices could happen at any lead label
+    #[serde(rename = "leads")]
+    LeadLabels,
+    /// Splice only happen whenever a call _could_ have happened
+    #[serde(rename = "call locations")]
+    CallLocations,
+    /// Splices only happen when calls are actually made
+    #[serde(rename = "calls")]
+    Calls,
+}
+
+impl Default for SpliceStyle {
+    fn default() -> Self {
+        Self::LeadLabels
+    }
+}
+
 /// A type of call (e.g. bob or single)
 #[derive(Debug, Clone)]
 pub struct CallType {
@@ -340,6 +362,9 @@ pub enum QueryUpdate {
 pub struct Progress {
     /// How many chunks have been expanded so far
     pub iter_count: usize,
+    /// How many comps have been generated so far
+    pub num_comps: usize,
+
     /// The current length of the A* queue
     pub queue_len: usize,
     /// The average length of a composition in the queue
@@ -352,6 +377,8 @@ impl Progress {
     /// The [`Progress`] made by a search which hasn't started yet
     pub const START: Self = Self {
         iter_count: 0,
+        num_comps: 0,
+
         queue_len: 0,
         avg_length: 0.0,
         max_length: 0,
